@@ -55,12 +55,21 @@ test_that("Data Input: data frame variable format", {
 #----
 # Detecting invalid input data: "validate" must be a logical vector
 #----
-test_that("Data Input: Validate must be logical vector of length 1", {
+test_that("Data Input: validate must be logical vector of length 1", {
   expect_error(psm_analysis(toocheap = 1, cheap = 2, expensive = 3, tooexpensive = 4, validate = "yes"))
   expect_error(psm_analysis(toocheap = 1, cheap = 2, expensive = 3, tooexpensive = 4, validate = c(TRUE, TRUE)))
 }
 )
 
+#----
+# Detecting invalid input data: "interpolate" must be a logical vector
+#----
+test_that("Data Input: interpolate must be logical vector of length 1", {
+  expect_error(psm_analysis(toocheap = 1, cheap = 2, expensive = 3, tooexpensive = 4, interpolate = "yes"))
+  expect_error(psm_analysis(toocheap = 1, cheap = 2, expensive = 3, tooexpensive = 4, interpolate = c(TRUE, TRUE)))
+}
+)
+
 #----
 # Detecting invalid input data: NMS using vectors
 #----
